from typing import TYPE_CHECKING

from django.conf import settings
from celery.result import AsyncResult

from .tasks import send_letter

if TYPE_CHECKING:
    from .entities import Letter

__all__ = (
    'BaseEmailBackend',
    'EmailBackend',
    'CeleryEmailBackend',
)


class BaseEmailBackend():
    def send(self, letter: 'Letter'):
        return NotImplemented


class EmailBackend(BaseEmailBackend):
    def send(self, letter: 'Letter') -> 'Letter':
        return letter._send()


class CeleryEmailBackend(BaseEmailBackend):
    """
    Send email use case. Used to send email, duh.
    """

    def send(self, letter: 'Letter') -> AsyncResult:
        """
        Calls `send_letter` task to send email.

        Args:
            letter (Optional[Letter]): Letter entity instance.

        Returns:
            Optional[Letter]: Letter instance or None
        """

        return send_letter.apply_async(
            args=(letter.object.id,),
            countdown=getattr(settings, "POSTIE_TASK_COUNTDOWN", 5)
        )



from typing import Optional

from django.conf import settings
from tilda_wrapper_api.client import Client
from postie.models import Template
from .utils import get_tilda_credentials


def get_template_by_page_id(page_id: int) -> Optional[Template]:
    template: Template = Template.objects.filter(
        translations__tilda_id=page_id
    ).prefetch_related('translations').first()

    if not template:
        return None

    # We have to return template with activated language for a given page_id
    for translation in template.translations.all():
        if translation.tilda_id == page_id:
            template.set_current_language(translation.language_code)

    return template


def get_tilda_page_html(page_id: int, credentials) -> str:
    client = Client(
        public=credentials.public_key, secret=credentials.private_key
    )

    page_data = client.get_page_full(page_id)

    return page_data.result.html


def update_template_tilda_data(template: Template, page_id: int):
    credentials = get_tilda_credentials()
    page_html = get_tilda_page_html(page_id, credentials)

    # NOTE: template must be with an activated translation that has
    # corresponding page_id, like it's done in `get_template_by_page_id`
    template.html = page_html
    template.save()


def update_page(project_id: int, page_id: int):
    template = get_template_by_page_id(page_id)

    if not template:
        raise ValueError(
            f'Template with tilda page_id {page_id} does not exists')

    update_template_tilda_data(template, page_id)



# Generated by Django 2.1 on 2020-07-04 16:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('postie', '0004_auto_20200323_1754'),
    ]

    operations = [
        migrations.AddField(
            model_name='templatetranslation',
            name='tilda_html',
            field=models.TextField(blank=True, editable=False, null=True, verbose_name='Tilda HTML'),
        ),
        migrations.AddField(
            model_name='templatetranslation',
            name='tilda_id',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Tilda id'),
        ),
    ]



from importlib import import_module

from typing import List
from codemirror2.widgets import AdminCodeMirrorEditor

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.handlers.wsgi import WSGIRequest

__all__ = (
    'get_protocol',
    'get_domain',
    'to_list',
    'to_str',
    'default_language',
    'get_html_field_widget',
)


def get_protocol() -> str:
    """
    Returns data transfer protocol name.
    The value is determined by bool variable 'USE_HTTPS' in settings.

    Returns:
        str: 'https' if 'USE_HTTPS' is True, otherwise - 'http'.
    """

    return 'https' if getattr(settings, 'USE_HTTPS', False) else 'http'


def get_domain(request: WSGIRequest=None) -> str:
    """
    Returns domain name this site.

    Args:
        request (WSGIRequest): Request.

    Returns:
        str: Domain name.
    """

    site_name = Site.objects.get_current(request).domain

    return site_name


def to_list(string: str) -> List[str]:
    """
    Transforms string to list.

    Args:
        string (str): String to transform.

    Returns:
        List[str]: List
    """

    return string.split(',')


def to_str(_list: List[str]) -> str:
    """
    Transforms list to str.

    Args:
        _list (List[str]): List to transform to str

    Returns:
        str: Transformed list
    """

    return ','.join(_list)


def default_language() -> str:
    """
    Returns default site language.

    Returns:
        str: Default site language
    """
    if hasattr(settings, 'PARLER_LANGUAGES') and settings.PARLER_LANGUAGES:
        return (
            settings.PARLER_LANGUAGES.get('default', {}).get('fallback')
        )
    else:
        return settings.LANGUAGE_CODE


def get_html_field_widget() ->object:
    """
    Get custom widget from settings

    Returns:
        object: HTML field widget
    """
    if hasattr(settings, 'POSTIE_HTML_ADMIN_WIDGET') and settings.POSTIE_HTML_ADMIN_WIDGET:
        custom_widget = getattr(
            import_module(settings.POSTIE_HTML_ADMIN_WIDGET['widget_module']),
            settings.POSTIE_HTML_ADMIN_WIDGET['widget']
        )
        return custom_widget(**settings.POSTIE_HTML_ADMIN_WIDGET.get('attrs', {}))

    return AdminCodeMirrorEditor(
        modes=['css', 'xml', 'javascript', 'htmlmixed']
    )



from unittest.mock import patch

from django import test
from django.conf import settings
from django.core import mail

from django_dynamic_fixture import G, N

from ..entities import Template, Letter
from ..models import (
    Template as TemplateModel, Letter as LetterModel
)
from postie.tasks import send_letter

__all__ = (
    'TemplateEntityTestCase',
)


class TestBackend:
    def send(self, letter):
        letter.message.send()


class TemplateEntityTestCase(test.TestCase):

    def test_render_template_method(self):
        template = TemplateModel(
            event='event',
            name='template',
            subject='Test {{ subject_var }} {{ global_var }}',
            html='Test {{ body_var }} {{ global_var }}',
            plain='Test {{ txt_var }} {{ global_var }}'
        )
        entity = Template(template)
        rendered = entity.render(
            {
                'subject_var': 'subject', 'global_var': 'global',
                'body_var': 'body', 'txt_var': 'txt'
            }
        )
        self.assertEqual(rendered.get('subject'), 'Test subject global')
        self.assertEqual(rendered.get('html'), 'Test body global')
        self.assertEqual(rendered.get('plain'), 'Test txt global')

    def test_from_event_method(self):
        template = TemplateModel.objects.create(
            event='event',
            name='template',
            subject='Test {{ subject_var }} {{ global_var }}',
            html='Test {{ body_var }} {{ global_var }}',
            plain='Test {{ txt_var }} {{ global_var }}'
        )

        entity = Template.from_event('event')
        self.assertIsInstance(entity, Template)
        self.assertEqual(entity.object, template)

    def test_legend(self):
        legend = '\n'.join(
            f'{{{{ {key} }}}}: {value}'
            for key, value in
            settings.POSTIE_TEMPLATE_CONTEXTS.get('1', {}).items()
        )
        G(TemplateModel, event='1')

        entity = Template(TemplateModel.objects.get(event='1'))

        self.assertEqual(legend, entity.legend)

    def test_new_letter(self):
        template = G(TemplateModel, event='1')
        template.set_current_language('en')
        template.subject = 'Test subject {{var1}}'
        template.html = 'Test'
        template.plain = 'Test'
        template.save()

        entity = Template(template)

        letter_entity = entity.new_letter(
            context={'var1': 'var 1', 'var2': 'var 2'},
            recipients=['test@test.test'],
            email_from='asd@asd.asd'
        )

        self.assertEqual(letter_entity.object.subject, 'Test subject var 1')

    def test_new_letter_without_email_from(self):
        template = G(TemplateModel, event='1')
        entity = Template(template)

        template.subject = "subject"
        template.html = "html"
        template.plain = "plain"
        template.save()

        letter_entity = entity.new_letter(
            context={'var1': 'var 1', 'var2': 'var 2'},
            recipients=['test@test.test'],
        )

        self.assertEqual(
            letter_entity.object.email_from, settings.DEFAULT_FROM_EMAIL
        )

    def test_send(self):
        letter = G(LetterModel)

        # No usecase
        with self.settings(POSTIE_INSTANT_SEND=True):
            Letter(letter).send()
            self.assertEqual(len(mail.outbox), 1)

        # Celery email backend
        with self.settings(POSTIE_INSTANT_SEND=False):
            with patch('postie.tasks.send_letter.apply_async'):
                Letter(letter).send()
                self.assertEqual(len(mail.outbox), 1)

        with patch('postie.tasks.send_letter.apply_async'):
            send_letter.apply_async(args=(letter.id+1))
            self.assertEqual(len(mail.outbox), 1)

        # TestBackend
        Letter(letter).send(TestBackend)
        self.assertEqual(len(mail.outbox), 2)



from django.http.response import HttpResponse

from .utils import get_tilda_credentials
from .tasks import handle_webhook_task


def webhook_receiver(request):
    credentials = get_tilda_credentials()

    public_key = request.GET.get('publickey')
    project_id = int(request.GET.get('projectid') or 0)
    page_id = int(request.GET.get('pageid') or 0)

    if credentials.public_key == public_key and credentials.project_id == project_id:
        handle_webhook_task.delay(project_id, page_id)
        return HttpResponse('ok')

    return HttpResponse()



from typing import Dict, Any, List, Optional

from .entities import Template, Letter

__all__ = (
    'send_mail',
)


def send_mail(
        event: str, recipients: List[str], context: Dict[Any, Any],
        from_email: Optional[str]=None, attachments: Optional[List[Dict]]=None,
        language: str=None, backend=None
    ) -> Letter:
    """Shortcut to send email.

    Args:
        event (str): Event to send email. Used to get template.
        recipients (List[str]): Recipients email list.
        context (Dict[any, any]): Email context.
        from_email (Optional[str]): Sender email address.
        attachments (Optional[List[str]]): Letter attachments.
        language (str): Letter language.

    Raises:
        ValueError: No template found for given "event"

    Returns:
        Letter: New Letter entity.
    """

    if attachments is None:
        attachments = []

    template = Template.from_event(event)
    letter = template.new_letter(
        context=context,
        recipients=recipients,
        email_from=from_email,
        attachments=attachments,
        language=language
    )

    letter.send(backend)

    return letter



from pathlib import Path

from django import test
from django.core import mail

from ..tasks import send_letter
from ..models import Letter, Attachment
from ..const import LETTER_STATUSES

__all__ = (
    'LetterSendTestCase'
)


class LetterSendTestCase(test.TestCase):
    def test_wrong_letter_raised_error(self):
        with self.assertRaises(ValueError):
            send_letter(2)

    def test_lettter_without_files_send(self):
        letter = Letter.objects.create(
            email_from='cyberbudy@gmail.com',
            subject='Subject',
            html='<b>Mail body</b>',
            plain='Mail body',
            recipients='cyberbudy@gmail.com'
        )

        send_letter(letter.id)
        letter.refresh_from_db()
        self.assertEqual(letter.status, LETTER_STATUSES.sent)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, letter.subject)

    def test_lettter_with_files_send(self):
        letter = Letter.objects.create(
            email_from='cyberbudy@gmail.com',
            subject='Subject',
            html='<b>Mail body</b>',
            plain='Mail body',
            recipients='cyberbudy@gmail.com'
        )
        Attachment(
            letter=letter,
            attachment=open(Path(__file__).parent / 'file_fixture.json')
        )

        send_letter(letter.id)
        letter.refresh_from_db()

        self.assertEqual(letter.status, LETTER_STATUSES.sent)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, letter.subject)



from decimal import Decimal

from django import test, forms
from django.core import mail

from django_dynamic_fixture import G

from ..forms import MailSendFormMixin
from ..models import Template

__all__ = (
    'MailSendFormMixinTestCase'
)


class TestForm(MailSendFormMixin, forms.Form):
    char_field = forms.CharField()
    decimal_field = forms.DecimalField()
    choices_field = forms.ChoiceField(
        choices=(
            (1, 'first'),
            (2, 'second'),
            (3, 'third'),
        )
    )

    def get_email_recipients(self):
        return ['test@test.test']


# Emulate model form
class SaveForm(forms.Form):
    def save(self, commit=True):
        return 'object is saved'
    

class TestSaveForm(TestForm, SaveForm):
    char_field = forms.CharField()
    decimal_field = forms.DecimalField()
    choices_field = forms.ChoiceField(
        choices=(
            (1, 'first'),
            (2, 'second'),
            (3, 'third'),
        )
    )
    
    def get_email_recipients(self):
        return ['test@test.test']


class MailSendFormMixinTestCase(test.TestCase):
    def test_get_email_event(self):
        form = MailSendFormMixin()
        self.assertEqual(form.get_email_template_event(), None)

        form.email_template_event = 'event'

        self.assertEqual(form.get_email_template_event(), 'event')

    def test_get_email_sender(self):
        form = MailSendFormMixin()

        with self.settings(DEFAULT_FROM_EMAIL='admin@admin.com'):
            self.assertEqual(
                form.get_email_sender(), 'admin@admin.com'
            )

    def test_get_email_recipients(self):
        form = MailSendFormMixin()

        with self.assertRaises(NotImplementedError):
            form.get_email_recipients()

    def test_get_email_context(self):
        form = TestForm(data={
            'char_field': 'char data',
            'decimal_field': Decimal(12.033),
            'choices_field': 1,
        })

        # Form should be validated first, before the context retrieving.
        with self.assertRaises(AttributeError):
            form.get_email_context()

        form.is_valid()

        self.assertEqual(
            form.get_email_context(),
            {
                'char_field': 'char data',
                'choices_field': '1',
                'decimal_field': str(Decimal(12.033))
            }
        )

    def test_send_email(self):
        template = G(Template)

        template.subject = "subject"
        template.html = "html"
        template.plain = "plain"
        template.save()

        form = TestForm(data={
            'char_field'   : 'char data',
            'decimal_field': Decimal(12.033),
            'choices_field': 1,
        })

        form.email_template_event = template.event
        form.is_valid()
        form.send_email()

        self.assertEqual(len(mail.outbox), 1)

    def test_save(self):
        # Test the form without save method is sending the email
        template = G(Template)

        template.subject = "subject"
        template.html = "html"
        template.plain = "plain"
        template.save()

        form = TestForm(data={
            'char_field'   : 'char data',
            'decimal_field': Decimal(12.033),
            'choices_field': 1,
        })

        form.email_template_event = template.event
        form.is_valid()
        form.save()

        self.assertEqual(len(mail.outbox), 1)

    def test_save_implemented(self):
        # Test the form with save method is sending the email

        template = G(Template)
        template.subject = "subject"
        template.html = "html"
        template.plain = "plain"
        template.save()

        form = TestSaveForm(data={
            'char_field'   : 'char data',
            'decimal_field': Decimal(12.033),
            'choices_field': 1,
        })

        form.email_template_event = template.event
        form.is_valid()
        response = form.save()

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(response, 'object is saved')



from django.utils.translation import pgettext_lazy
from django.db import models
from solo.models import SingletonModel


class Preferences(SingletonModel):
    public_key = models.CharField(
        verbose_name=pgettext_lazy('tilda preferences model', 'Public key'),
        max_length=255
    )
    private_key = models.CharField(
        verbose_name=pgettext_lazy('tilda preferences model', 'Private key'),
        max_length=255
    )
    project_id = models.PositiveIntegerField(
        verbose_name=pgettext_lazy('tilda preferences model', 'Project id'),
        null=True
    )

    class Meta:
        verbose_name = pgettext_lazy('tilda preferences model', 'Preferences')



# Generated by Django 2.0.2 on 2018-05-01 06:10

import ckeditor.fields
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import parler.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('attachment', models.FileField(upload_to='', verbose_name='Letter attachment')),
            ],
            options={
                'verbose_name': 'Letter attachment',
                'verbose_name_plural': 'Letter attachments',
            },
        ),
        migrations.CreateModel(
            name='Letter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('subject', models.CharField(max_length=255, verbose_name='Subject')),
                ('html', ckeditor.fields.RichTextField(verbose_name='HTML')),
                ('plain', models.TextField(verbose_name='Plain text')),
                ('email_from', models.EmailField(blank=True, max_length=255, null=True, verbose_name='Email from')),
                ('recipients', models.TextField(verbose_name='Letter recipients')),
                ('event', models.CharField(blank=True, max_length=255, null=True, verbose_name='Event')),
                ('status', models.CharField(choices=[('draft', 'draft'), ('sent', 'Sent'), ('failed', 'Failed')], default='draft', max_length=255, verbose_name='Status')),
                ('language', models.CharField(blank=True, max_length=127, null=True, verbose_name='Language')),
            ],
            options={
                'verbose_name': 'Mail letter',
                'verbose_name_plural': 'Mail letters',
            },
        ),
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('message', models.CharField(max_length=255, verbose_name='Log message')),
                ('traceback', models.TextField(blank=True, null=True, verbose_name='Traceback')),
                ('letter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='postie.Letter', verbose_name='Letter')),
            ],
            options={
                'verbose_name': 'Log',
                'verbose_name_plural': 'Logs',
            },
        ),
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('event', models.CharField(choices=[('1', '1'), ('2', '2'), ('3', '3')], max_length=255, verbose_name='Event')),
            ],
            options={
                'verbose_name': 'Mail template',
                'verbose_name_plural': 'Mail templates',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='TemplateTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('subject', models.CharField(max_length=255, verbose_name='Subject')),
                ('html', models.TextField(verbose_name='HTML text')),
                ('plain', models.TextField(blank=True, help_text='Alternative version of html text', null=True, verbose_name='Plain text')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='postie.Template')),
            ],
            options={
                'verbose_name': 'Mail template Translation',
                'db_table': 'postie_template_translation',
                'db_tablespace': '',
                'managed': True,
                'default_permissions': (),
            },
        ),
        migrations.AddField(
            model_name='attachment',
            name='letter',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='postie.Letter', verbose_name='Letter'),
        ),
        migrations.AlterUniqueTogether(
            name='templatetranslation',
            unique_together={('language_code', 'master')},
        ),
    ]



# Generated by Django 2.0.2 on 2018-05-01 08:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('postie', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='letter',
            name='recipients',
            field=models.TextField(help_text='Enter email address separated by commas', verbose_name='Letter recipients'),
        ),
    ]



import traceback
from typing import Dict, Any, List, Optional, Union

from django.template import Context, Template as DjangoTemplate
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.utils.translation import ugettext_lazy as _
from django.db import models

from .models import (
    Template as TemplateModel, Attachment, Letter as LetterModel,
    Log as LogModel
)
from .utils import to_str, to_list, default_language
from .const import LETTER_STATUSES, POSTIE_TEMPLATE_CONTEXTS
from .backends import CeleryEmailBackend, EmailBackend

__all__ = (
    'Letter',
    'Template'
)


class BaseEntity:
    """
    Base entity class.

    Attributes:
        model (models.Model): Entity model.
        object: Entity instance.
    """

    model = None  # type: models.Model

    def __init__(self, obj):
        self.object = obj


class Letter(BaseEntity):
    """
    Letter entity. Stores letter mailing logic.
    """

    model = LetterModel  # type: LetterModel

    @property
    def message(self) -> EmailMultiAlternatives:
        """
        Creates corresponding EmailMultiAlternatives letter.

        Returns:
            EmailMultiAlternatives
        """

        message = EmailMultiAlternatives(
            self.object.subject, self.object.plain, self.object.email_from,
            to_list(self.object.recipients)
        )

        if self.object.html:
            message.attach_alternative(self.object.html, "text/html")

        for attachment in self.object.attachments.all():
            message.attach_file(
                f'{settings.MEDIA_ROOT}/{attachment.attachment.name}'
            )

        return message

    @property
    def backend(self) -> Union[EmailBackend, CeleryEmailBackend]:
        if settings.POSTIE_INSTANT_SEND:
            return EmailBackend

        return CeleryEmailBackend

    def _send(self):
        """
        Actual sending method. Message is generated here and send using
        django's email backend specified in the settings.
        """

        try:
            self.message.send()
            is_failed = False
        except Exception as e:
            LogModel.objects.create(
                letter=self.object,
                message=str(e),
                traceback=traceback.format_exc()
            )
            is_failed = True

        if is_failed:
            self.set_failed()
        else:
            self.set_sent()

    def send(self, backend: Optional[CeleryEmailBackend] = None) -> None:
        """
        Sends an email letter using the corresponding UseCase class.
        If not backend specified SendMailUseCase is used.

        Args:
            backend (Optional[SendMailUseCase]):

        Returns:
            None
        """

        if not backend:
            backend = self.backend

        backend().send(self)

    def set_failed(self) -> None:
        """
        Sets the letter as a failed.

        Returns:
            None
        """

        self.object.status = LETTER_STATUSES.failed
        self.object.save()

    def set_sent(self) -> None:
        """
        Sets the letter a sent.

        Returns:
            None
        """

        self.object.status = LETTER_STATUSES.sent
        self.object.save()

    def add_attachment(self, file_name: str, attachment) -> None:
        """
        Adds attachment to a letter.

        Args:
            file_name (str): File name.
            attachment: File content.

        Returns:
            None
        """

        obj = Attachment(letter=self.object)
        obj.attachment.save(file_name, attachment, save=True)

    @classmethod
    def load_from_id(cls, pk: int) -> 'Letter':
        """
        Returns Letter entity loaded from DB with a given ID.

        Args:
            pk (int): LetterModel id.

        Returns:
            Letter: Letter entity instance.
        """

        orm_letter = LetterModel.objects.filter(pk=pk).not_sent().first()

        if not orm_letter:
            raise ValueError(
                _('Letter with id "{}" is not exists or already sent')
                .format(pk)
            )

        return Letter(orm_letter)


class Template(BaseEntity):
    """
    Template entity. Stores logic for a creating letter from a
    template.
    """

    model = TemplateModel  # type: TemplateModel

    @property
    def legend(self) -> str:
        """
        Returns prettified template legend.

        Returns:
            str: Template variable language.
        """

        legend = '\n'.join(
            f'{{{{ {key} }}}}: {value}'
            for key, value in
            POSTIE_TEMPLATE_CONTEXTS.get(self.object.event, {}).items()
        )

        return legend

    def set_language(self, language: str = None) -> None:
        """
        Changes object language.

        Args:
            language (str): Language to set.
        """

        if not language:
            language = default_language()

        if self.object.get_current_language() != language:
            if not self.object.has_translation(language):
                raise ValueError(
                    _('There is not translation for "{}" for {} template')
                    .format(language, self.object.name)
                )

            self.object.set_current_language(language, initialize=True)

    def render(self, context: Dict[str, Any]) -> Dict[str, str]:
        """
        Renders template with a given context.

        Args:
            context (Dict[str, Any]): Template context.

        Returns:
            Dict[str, str]: Rendered template fields to use in letter.
        """

        subject = DjangoTemplate(self.object.subject).render(Context(context))
        html = DjangoTemplate(self.object.get_html()).render(Context(context))

        if self.object.tilda_id:
            plain = html
        else:
            plain = DjangoTemplate(self.object.plain).render(Context(context))

        return {
            'subject': subject,
            'html': html,
            'plain': plain
        }

    def new_letter(
        self,
        context: Dict[str, Any],
        recipients: List[str],
        email_from: Optional[str] = None,
        attachments: Optional[List] = None,
        language: Optional[str] = None
    ) -> Letter:
        """Creates new letter

        Args:
            context (Dict[str, Any]): Mail letter context.
            recipients (List[str]): Letter recipients.
            email_from (Optional[str]): Letter sender email address.
            attachments (Optional[List]): Letter attachments.
            language (Optional[str]): Letter language to send.

        Returns:
            Letter: New Letter entity
        """

        self.set_language(language)

        rendered_fields = self.render(context)

        if not email_from:
            email_from = settings.DEFAULT_FROM_EMAIL

        letter = LetterModel.objects.create(
            email_from=email_from,
            event=self.object.event,
            recipients=to_str(recipients),
            language=language,
            **rendered_fields
        )
        letter_entity = Letter(letter)

        if attachments:
            for attachment in attachments:
                letter_entity.add_attachment(
                    *list(attachment.items())[0]
                )

        return letter_entity

    @classmethod
    def from_event(cls, event: str) -> 'Template':
        """
        Returns Template entity for the given event.

        Args:
            event (str): Template event to search.

        Returns:
            Template: Template entity instance.
        """

        template = TemplateModel.objects.filter(event=event).first()

        if not template:
            raise ValueError(_('No MailTemplate for "{}" event').format(event))

        return cls(template)



from ckeditor.fields import RichTextField
from django.db import models
from django.utils.translation import ugettext_lazy as _
from model_utils.models import TimeStampedModel
from parler.models import TranslatableModel, TranslatedFields

from .const import EVENTS, LETTER_STATUSES
from .querysets import LetterQuerySet

__all__ = (
    'Template',
    'Letter',
    'Attachment',
    'Log',
)


class Template(TranslatableModel):
    """
    Template model.

    Attributes:
        name (models.CharField): Template system name.
        event (models.CharField): Template event to send email on.
        subject (models.CharField): Letter subject.
        html (RichTextField) Letter html text version.
        plain (mdoels.
    """

    name = models.CharField(verbose_name=_('Name'), max_length=255)

    event = models.CharField(
        verbose_name=_('Event'), max_length=255, choices=EVENTS
    )

    translations = TranslatedFields(
        subject=models.CharField(
            verbose_name=_('Subject'),
            help_text=_('Write variables inside {{  }}.'),
            max_length=255, null=True
        ),
        html=models.TextField(
            verbose_name=_('HTML text'),
            help_text=_('Write variables inside {{  }}.'),
            blank=True, null=True
        ),
        plain=models.TextField(
            verbose_name=_('Plain text'),
            help_text=_(
                'Alternative version of html text.'
                'Write variables inside {{  }}.',
            ),
            blank=True, null=True
        ),
        # Tilda integration fields
        tilda_id=models.PositiveIntegerField(
            verbose_name=_('Tilda id'), blank=True, null=True,
            help_text=_(
                'If not empty `HTML` field will be not used anymore, '
                'intead tilda page will be'
            )
        ),
        tilda_html=models.TextField(
            verbose_name=_('Tilda HTML'), editable=False, blank=True,
            null=True
        )
    )

    class Meta:
        verbose_name = _('Mail template')
        verbose_name_plural = _('Mail templates')

    def __str__(self) -> str:
        return self.name

    def get_html(self) -> str:
        if self.tilda_id:
            return self.tilda_html or self.html

        return self.html


class Letter(TimeStampedModel, models.Model):
    """Mail letter model.

    Attributes:
        subject (CharField): Letter's subject.
        html (RichTextField): Letter's html text.
        text (TextField): Letter's plain text
        email_from (EmailField): Letter's email from(sender).
        recipients (TextField): Letter's list of recipients.
        event (CharField): Letter's event.
        language (CharField): Letter's language was send.
        status (CharField): Letter's send status.
        created (DateTimeField): Log's creation timestamp.
        modified (DateTimeField): Log's updating timestamp.
    """

    subject = models.CharField(max_length=255, verbose_name=_('Subject'))
    html = RichTextField(verbose_name=_('HTML'), blank=True, null=True)
    plain = models.TextField(
        verbose_name=_('Plain text'), blank=True, null=True
    )

    email_from = models.EmailField(
        verbose_name=_('Email from'),
        max_length=255, blank=True, null=True
    )
    recipients = models.TextField(
        verbose_name=_('Letter recipients'),
        help_text=_('Enter email address separated by commas')
    )
    event = models.CharField(
        verbose_name=_('Event'), max_length=255, blank=True, null=True
    )

    status = models.CharField(
        verbose_name=_('Status'), max_length=255,
        choices=LETTER_STATUSES, default=LETTER_STATUSES.draft
    )
    language = models.CharField(
        verbose_name=_('Language'), max_length=127, blank=True, null=True
    )

    objects = LetterQuerySet.as_manager()

    class Meta:
        verbose_name = _('Mail letter')
        verbose_name_plural = _('Mail letters')

    def __str__(self) -> str:
        return f'{self.subject}'


class Attachment(TimeStampedModel, models.Model):
    """Letter attachment.

    Attributes:
        letter (ForeignKey): Attachment's letter relation.
        attachment (FileField): Attachment's file.
        created (DateTimeField): Log's creation timestamp.
        modified (DateTimeField): Log's updating timestamp.
    """

    letter = models.ForeignKey(
        Letter, verbose_name=_('Letter'), on_delete=models.CASCADE,
        related_name='attachments'
    )

    attachment = models.FileField(verbose_name=_('Letter attachment'))

    class Meta:
        verbose_name = _('Letter attachment')
        verbose_name_plural = _('Letter attachments')


class Log(TimeStampedModel, models.Model):
    """Mail log.

    Attributes:
        message (CharField): Log's message.
        traceback (TextField): Log's detailed traceback.
        letter (ForeignKey): Log's letter relation.
        created (DateTimeField): Log's creation timestamp.
        modified (DateTimeField): Log's updating timestamp.
    """

    message = models.CharField(verbose_name=_('Log message'), max_length=255)
    traceback = models.TextField(
        verbose_name=_('Traceback'), blank=True, null=True
    )

    letter = models.ForeignKey(
        Letter, verbose_name=_('Letter'), on_delete=models.CASCADE,
        related_name='letter_logs'
    )

    class Meta:
        verbose_name = _('Log')
        verbose_name_plural = _('Logs')

    def __str__(self) -> str:
        return f'{self.message} at {self.created}'



from django import test

from django_dynamic_fixture import G

from ..const import LETTER_STATUSES
from ..models import Letter, Template, Log

__all__ = (
    'LetterTestCase',
)


class LetterTestCase(test.TestCase):
    def test_sent_query_method(self):
        G(Letter, status=LETTER_STATUSES.draft, n=5)
        G(Letter, status=LETTER_STATUSES.sent, n=5)
        G(Letter, status=LETTER_STATUSES.failed, n=5)

        self.assertEqual(Letter.objects.sent().count(), 5)

    def test_not_sent_query_method(self):
        G(Letter, status=LETTER_STATUSES.draft, n=5)
        G(Letter, status=LETTER_STATUSES.sent, n=5)
        G(Letter, status=LETTER_STATUSES.failed, n=5)

        self.assertEqual(Letter.objects.sent().count(), 5)
    
    def test_str(self):
        letter = G(Letter)
        
        self.assertIsInstance(str(letter), str)


class TemplateTestCase(test.TestCase):
    def test_str(self):
        template = G(Template)
        
        self.assertIsInstance(str(template), str)


class LogTestCase(test.TestCase):
    def test_str(self):
        log = G(Log)
        
        self.assertIsInstance(str(log), str)



# Generated by Django 2.1 on 2020-03-23 17:54

import ckeditor.fields
from django.db import migrations, models
import django.db.models.deletion
import parler.fields


class Migration(migrations.Migration):

    dependencies = [
        ('postie', '0003_auto_20180504_0340'),
    ]

    operations = [
        migrations.AlterField(
            model_name='letter',
            name='html',
            field=ckeditor.fields.RichTextField(blank=True, null=True, verbose_name='HTML'),
        ),
        migrations.AlterField(
            model_name='letter',
            name='plain',
            field=models.TextField(blank=True, null=True, verbose_name='Plain text'),
        ),
        migrations.AlterField(
            model_name='log',
            name='letter',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='letter_logs', to='postie.Letter', verbose_name='Letter'),
        ),
        migrations.AlterField(
            model_name='templatetranslation',
            name='html',
            field=models.TextField(blank=True, help_text='Write variables inside {{  }}.', null=True, verbose_name='HTML text'),
        ),
        migrations.AlterField(
            model_name='templatetranslation',
            name='master',
            field=parler.fields.TranslationsForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='postie.Template'),
        ),
        migrations.AlterField(
            model_name='templatetranslation',
            name='plain',
            field=models.TextField(blank=True, help_text='Alternative version of html text.Write variables inside {{  }}.', null=True, verbose_name='Plain text'),
        ),
        migrations.AlterField(
            model_name='templatetranslation',
            name='subject',
            field=models.CharField(help_text='Write variables inside {{  }}.', max_length=255, null=True, verbose_name='Subject'),
        ),
    ]



from pathlib import Path

from django import test
from django.core import mail
from django.core.mail.message import EmailMultiAlternatives
from django_dynamic_fixture import G

from ..shortcuts import send_mail
from ..models import Template


__all__ = (
    'SendMailTestCase'
)


class SendMailTestCase(test.TestCase):
    def setUp(self):
        self.new_events = (
            'event', '1'
        )

    def test_wrong_template_raised_error(self):
        with self.assertRaises(ValueError):
            send_mail(2, recipients=[], context={})

    def test_mail_without_files_send(self):
        with self.settings(POSTIE_TEMPLATE_CHOICES=self.new_events, POSTIE_INSTANT_SEND=True):
            template = G(
                Template,
                name='event',
                event='event',
            )
            template.html = '<b>Mail body</b>'
            template.plain = 'Mail body'
            template.subject = 'Subject'
            template.save()

            send_mail(
                event='event',
                from_email='cyberbudy@gmail.com',
                recipients=['cyberbudy@gmail.com'],
                context={}
            )

            self.assertEqual(len(mail.outbox), 1)
            self.assertEqual(mail.outbox[0].subject, 'Subject')

    def test_mail_with_files_send(self):
        with self.settings(POSTIE_TEMPLATE_CHOICES=self.new_events,
                           POSTIE_INSTANT_SEND=True):
            template = G(
                Template,
                name='event',
                event='event',
            )
            template.html = '<b>Mail body</b>'
            template.plain = 'Mail body'
            template.subject = 'Subject'
            template.save()

            with open(Path(__file__).parent / 'file_fixture.json') as f:
                send_mail(
                    event='event',
                    from_email='cyberbudy@gmail.com',
                    recipients=['cyberbudy@gmail.com'],
                    context={},
                    attachments=[{
                        'file_fixtures': f
                    }]
                )

            self.assertEqual(len(mail.outbox), 1)
            self.assertEqual(mail.outbox[0].subject, 'Subject')

    def test_mail_with_unknown_language(self):
        with self.settings(POSTIE_TEMPLATE_CHOICES=self.new_events,
                           POSTIE_INSTANT_SEND=True):
            template = G(
                Template,
                name='event',
                event='event',
            )
            template.html = '<b>Mail body</b>'
            template.plain = 'Mail body'
            template.subject = 'Subject'
            template.save()

            with self.assertRaises(ValueError):
                send_mail(
                    event='event',
                    from_email='cyberbudy@gmail.com',
                    recipients=['cyberbudy@gmail.com'],
                    context={},
                    language='asd'
                )

    def test_mail_without_parler_settings(self):
        with self.settings(
            POSTIE_TEMPLATE_CHOICES=self.new_events,
            POSTIE_INSTANT_SEND=True,
            PARLER_LANGUAGES={},
            LANGUAGES=(('en', 'EN'),),
            LANGUAGE_CODE='en'
        ):
            template = G(
                Template,
                name='event',
                event='event',
            )
            template.subject = 'Subject'
            template.html = '<b>Mail body</b>'
            template.plain = 'Mail body'
            template.save()

            send_mail(
                event='event',
                from_email='cyberbudy@gmail.com',
                recipients=['cyberbudy@gmail.com'],
                context={}
            )
            self.assertEqual(len(mail.outbox), 1)
            self.assertEqual(mail.outbox[0].subject, 'Subject')

    def test_mail_with_translated_language(self):
        with self.settings(
            POSTIE_TEMPLATE_CHOICES=self.new_events,
            POSTIE_INSTANT_SEND=True,
        ):
            template = G(
                Template,
                name='event',
                event='event',
            )
            template.set_current_language('en')
            template.subject = 'Test Subject 1'
            template.html = 'Test html 1'
            template.plain = 'Test plain 1'
            template.save()

            send_mail(
                event='event',
                from_email='cyberbudy@gmail.com',
                recipients=['cyberbudy@gmail.com'],
                context={}
            )

            self.assertEqual(len(mail.outbox), 1)
            self.assertEqual(mail.outbox[0].subject, 'Test Subject 1')

    def test_mail_with_tilda_template(self):
        with self.settings(
            POSTIE_TEMPLATE_CHOICES=self.new_events,
            POSTIE_INSTANT_SEND=True,
        ):
            template = G(
                Template,
                name='event',
                event='event',
            )
            template.set_current_language('en')
            template.html = 'Test html 1'
            template.plain = 'Test plain 1'
            template.subject = 'Test Subject 1'
            template.tilda_id = 1
            template.tilda_html = 'TILDA HTMl'
            template.save()

            send_mail(
                event='event',
                from_email='cyberbudy@gmail.com',
                recipients=['cyberbudy@gmail.com'],
                context={}
            )

            self.assertEqual(template.tilda_id, 1)
            self.assertEqual(len(mail.outbox), 1)
            self.assertEqual(mail.outbox[0].subject, 'Test Subject 1')
            self.assertEqual(mail.outbox[0].body, 'TILDA HTMl')



from unittest import mock
from dataclasses import dataclass

from ckeditor.widgets import CKEditorWidget
from codemirror2.widgets import AdminCodeMirrorEditor
from django.test import TestCase, override_settings
from django.conf import settings
from django.urls import reverse
from django.utils.translation import activate
from django_dynamic_fixture import G, N
from parler.utils.context import switch_language

from postie.models import Template as TemplateModel
from postie.entities import Template
from postie.integrations.tilda.models import Preferences
from postie.integrations.tilda.tasks import handle_webhook_task
from postie.integrations.tilda import services, utils


__all__ = (
    'TildaTemplateTestCase',
)


class TildaTemplateTestCase(TestCase):
    def test_template_with_tilda_returns_proper_html(self):
        activate('it')
        template = N(TemplateModel)
        template.tilda_id = 123
        template.html = "Html"
        template.tilda_html = "Tilda html"
        self.assertEqual(template.get_current_language(), 'it')

        self.assertEqual(template.get_html(), "Tilda html")

        entity = Template(template)
        self.assertEqual(entity.render({})['html'], "Tilda html")

        template.tilda_id = None

        self.assertEqual(template.get_html(), "Html")
        entity = Template(template)
        self.assertEqual(entity.render({})['html'], "Html")

        with switch_language(template, 'en'):
            template.tilda_id = 456
            template.tilda_html = "en html"
            self.assertEqual(template.get_current_language(), 'en')
            self.assertEqual(template.get_html(), "en html")
            entity = Template(template)
            self.assertEqual(entity.render({})['html'], "en html")


class TildaHookTestCase(TestCase):
    @override_settings(CELERY_TASK_ALWAYS_EAGER=True)
    @mock.patch('postie.integrations.tilda.tasks.handle_webhook_task.delay')
    def test_hoop_view_response(self, mock):
        preferences = G(
            Preferences,
            public_key="public",
            private_key="private_key",
            project_id=1
        )
        response = self.client.get(reverse('webhook_receiver'))

        self.assertEqual(response.content.decode(), "")
        self.assertEqual(mock.called, False)

        preferences = G(
            Preferences,
            public_key="public",
            private_key="private_key",
            project_id=1
        )
        response = self.client.get(
            reverse('webhook_receiver'),
            {"publickey": "public", "projectid": 1, 'pageid': 1}
        )

        self.assertEqual(response.content.decode(), "ok")
        self.assertEqual(mock.called, True)


@dataclass
class MockPageResult:
    html: str


@dataclass
class MockPage:
    result: MockPageResult


class TildaServicesTestCase(TestCase):
    def test_get_template_by_page_id(self):
        activate('en')
        self.assertIsNone(services.get_template_by_page_id(1))

        template = G(TemplateModel)
        with switch_language(template, 'en'):
            template.tilda_id = 1
            template.html = "EN"
            template.save()

        self.assertEqual(services.get_template_by_page_id(1).id, template.id)
        self.assertEqual(services.get_template_by_page_id(1).html, "EN")

        activate('it')
        with switch_language(template, 'it'):
            template.tilda_id = 2
            template.html = "IT"
            template.save()

        self.assertEqual(services.get_template_by_page_id(2).id, template.id)
        self.assertEqual(services.get_template_by_page_id(2).html, "IT")

        self.assertEqual(services.get_template_by_page_id(1).id, template.id)
        self.assertEqual(services.get_template_by_page_id(1).html, "EN")

    @mock.patch('tilda_wrapper_api.client.Client.get_page_full', return_value=MockPage(result=MockPageResult('HTML')))
    def test_get_tilda_page_html(self, get_page_full_mock):
        preferences = G(
            Preferences,
            public_key="public",
            private_key="private_key",
            project_id=1
        )

        self.assertEqual(
            services.get_tilda_page_html(1, utils.get_tilda_credentials()),
            'HTML'
        )

    @mock.patch('tilda_wrapper_api.client.Client.get_page_full')
    def test_update_template_tilda_data(self, get_page_full_mock):
        preferences = G(
            Preferences,
            public_key="public",
            private_key="private_key",
            project_id=1
        )

        template = G(TemplateModel)
        template.set_current_language('en')
        template.tilda_id = 1
        template.save()

        get_page_full_mock.return_value = MockPage(
            result=MockPageResult('HTML')
        )

        services.update_template_tilda_data(template, 1)
        self.assertEqual(services.get_template_by_page_id(1).html, 'HTML')

        template.set_current_language('it')
        template.tilda_id = 2
        template.save()

        get_page_full_mock.return_value = MockPage(
            result=MockPageResult('IT HTML')
        )

        services.update_template_tilda_data(template, 2)
        self.assertEqual(services.get_template_by_page_id(2).html, 'IT HTML')
        self.assertEqual(services.get_template_by_page_id(1).html, 'HTML')


class TildaTaskTestCase(TestCase):
    def test_handle_webhook_task(self):
        preferences = G(
            Preferences,
            public_key="public",
            private_key="private_key",
            project_id=1
        )

        with self.assertRaises(ValueError):
            handle_webhook_task(1, 1)



# Generated by Django 2.0.2 on 2018-05-04 03:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('postie', '0002_auto_20180501_0834'),
    ]

    operations = [
        migrations.AlterField(
            model_name='templatetranslation',
            name='html',
            field=models.TextField(null=True, verbose_name='HTML text'),
        ),
        migrations.AlterField(
            model_name='templatetranslation',
            name='subject',
            field=models.CharField(max_length=255, null=True, verbose_name='Subject'),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import pgettext_lazy


class PostieTildaConfig(AppConfig):
    name = 'postie.integrations.tilda'
    verbose_name = pgettext_lazy(
        'Postie integrations tilda app', 'Postie tilda'
    )



from smtplib import SMTPException
from unittest import mock

from django import test
from django.core import mail
from django.test import override_settings
from django_dynamic_fixture import G

from ..models import Letter as LetterModel, Log
from ..entities import Letter
from ..const import LETTER_STATUSES


@override_settings(
    EMAIL_BACKEND='django.core.mail.backends.locmem.EmailBackend'
)
class ValidMailLetterTestCase(test.TestCase):
    model = LetterModel

    def test_send_email_logs_none(self):
        letter = G(
            self.model,
            subject='Text',
            plain='Text',
            recipients='user@gmail.com'
        )
        Letter(letter).send()

        self.assertEqual(Log.objects.count(), 0)

    def test_send_email_letter_status_sent(self):
        letter = G(
            self.model,
            subject='Text',
            plain='Text',
            recipients='user@gmail.com'
        )
        Letter(letter).send()
        letter.refresh_from_db()
        self.assertEqual(letter.status, LETTER_STATUSES.sent)

    def test_send_email_one_receiver_recipients(self):
        letter = G(
            self.model,
            subject='Text',
            plain='<b>Text</b>',
            recipients='user@gmail.com'
        )
        Letter(letter).send()

        self.assertEqual(mail.outbox[0].to[0], 'user@gmail.com')

    def test_send_email_many_receivers_recipients(self):
        letter = G(
            self.model,
            subject='Text',
            plain='<b>Text</b>',
            recipients='user@gmail.com,admin@gmail.com'
        )
        Letter(letter).send()

        self.assertListEqual(
            mail.outbox[0].to, ['user@gmail.com', 'admin@gmail.com']
        )


class FailMailLetterTestCase(test.TestCase):
    model = LetterModel

    def setUp(self):
        self.patcher = mock.patch('django.core.mail.EmailMultiAlternatives.send')
        self.addCleanup(self.patcher.stop)

        self.mock = self.patcher.start()
        self.mock.side_effect = SMTPException('Send message failed!')

    def test_send_email_letter_status_fail(self):
        letter = G(
            self.model,
            subject='Text',
            plain='<b>Text</b>',
            recipients='user@gmail.com'
        )
        Letter(letter).send()
        letter.refresh_from_db()
        self.assertEqual(letter.status, LETTER_STATUSES.failed)

    def test_send_email_log_created(self):
        letter = G(
            self.model,
            subject='Text',
            plain='<b>Text</b>',
            recipients='user@gmail.com'
        )
        Letter(letter).send()

        self.assertEqual(Log.objects.count(), 1)




from unittest.mock import patch

from django import test
from django.core import mail

from django_dynamic_fixture import G

from ..backends import CeleryEmailBackend, EmailBackend
from ..models import Letter as LetterModel
from ..entities import Letter

__all__ = (
    'TestSendMailUseCase',
)


class TestSendMailUseCase(test.TestCase):
    def test_instant_send(self):
        letter = G(LetterModel)
        entity = Letter(letter)

        with self.settings(POSTIE_INSTANT_SEND=True):
            EmailBackend().send(entity)
            self.assertEqual(len(mail.outbox), 1)

    @test.override_settings(POSTIE_INSTANT_SEND=False)
    def test_celery_send(self):
        letter = G(LetterModel)
        entity = Letter(letter)

        with patch('postie.tasks.send_letter.apply_async'):
            CeleryEmailBackend().send(entity)
            self.assertEqual(len(mail.outbox), 0)



from celery import shared_task

__all__ = (
    'send_letter'
)


@shared_task(bind=True, default_retry_delay=30, max_retries=15,)
def send_letter(self, letter_id: int):
    """
    Right now this task is used to send letter. It's the only
    difference between `EmailBackend` and `CeleryEmailBackend` in how
    they send emails

    Args:
        letter_id (int): Letter object id

    """
    from .entities import Letter

    try:
        letter = Letter.load_from_id(letter_id)
        letter._send()
    except Exception as e:
        raise self.retry(exc=e, countdown=2 ** self.request.retries)



from django.db import models

from .const import LETTER_STATUSES

__all__ = (
    'LetterQuerySet',
)


class LetterQuerySet(models.QuerySet):
    def not_sent(self) -> models.QuerySet:
        """
        Filters not sent letters.
        
        Returns:
            models.QuerySet: not sent letters.
        """
        return self.exclude(status=LETTER_STATUSES.sent)

    def sent(self) -> models.QuerySet:
        """
        Filters sent letters
        
        Returns:
            models.QuerySet: sent letters.
        """
        return self.filter(status=LETTER_STATUSES.sent)



from django.conf import settings
from django.urls import path, include
from django.contrib.staticfiles.urls import static
from django.contrib import admin


urlpatterns = [
    path('admin/', admin.site.urls),
    path('', include('postie.integrations.tilda.urls'))
]

if settings.DEBUG:
    urlpatterns += (
        static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT) +
        static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
    )



from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from parler.admin import TranslatableAdmin
from parler.forms import TranslatableModelForm

from .utils import get_html_field_widget
from .models import Attachment, Log
from .entities import Template, Letter


__all__ = (
    'TemplateAdmin',
    'LetterAdmin'
)


class TemplateForm(TranslatableModelForm):
    html = forms.CharField(
        widget=get_html_field_widget(), required=False
    )
    context = forms.CharField(
        label=_('Context'), widget=forms.Textarea(), disabled=True,
        required=False
    )

    class Meta:
        model = Template.model
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if self.instance:
            self.fields['context'].initial = Template(self.instance).legend


class AddTemplateForm(TranslatableModelForm):
    class Meta:
        model = Template.model
        fields = ['name', 'event']


@admin.register(Template.model)
class TemplateAdmin(TranslatableAdmin):
    """
    Admin interface for mail.
    """
    list_display = ['event', 'subject']
    form = TemplateForm
    add_form = AddTemplateForm
    fieldsets = (
        (None, {
            'fields': [
                'name', 'event',
            ]
        }),
        (_('Template'), {
            'fields': [
                'subject', 'html', 'context', 'plain'
            ]
        }),
    )
    tilda_fieldsets = (
        (None, {
            'fields': [
                'name', 'event',
            ]
        }),
        (_('Template'), {
            'fields': [
                'subject', 'html', 'context', 'plain'
            ]
        }),
        (_('Integrations'), {
            'fields': [
                'tilda_id'
            ]
        })
    )
    add_fieldsets = (
        (None, {
            'fields': [
                'name', 'event'
            ]
        }),
    )

    def get_fieldsets(self, request, obj=None):
        if not obj:
            return self.add_fieldsets

        if 'postie.integrations.tilda' in settings.INSTALLED_APPS:
            return self.tilda_fieldsets

        return super().get_fieldsets(request, obj)

    def get_form(self, request, obj=None, **kwargs):
        """
        Use special form during user creation
        """
        defaults = {}

        if obj is None:
            defaults['form'] = self.add_form

        defaults.update(kwargs)

        return super().get_form(request, obj, **defaults)


class AttachmentInline(admin.TabularInline):
    model = Attachment
    extra = 0


class LogInline(admin.StackedInline):
    model = Log
    extra = 0

    def has_add_permission(self, *args, **kwargs):
        return False

    def has_change_permission(self, *args, **kwargs):
        return False


def send_letters(modeladmin, request, queryset) -> None:
    """
    Sends selected letters.
    """

    for letter in queryset:
        Letter(letter).send()


send_letters.short_description = "Send letters"


@admin.register(Letter.model)
class LetterAdmin(admin.ModelAdmin):
    """
    Admin interface for mail.
    """

    list_display = ['subject', 'status', 'event', 'created']
    list_filter = ['status', 'event']
    search_fields = ['subject', 'recipients']

    inlines = [AttachmentInline, LogInline]
    actions = [send_letters]



# Generated by Django 2.1 on 2020-07-04 17:13

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Preferences',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('public_key', models.CharField(max_length=255, verbose_name='Public key')),
                ('private_key', models.CharField(max_length=255, verbose_name='Private key')),
            ],
            options={
                'verbose_name': 'Preferences',
            },
        ),
    ]



from ckeditor.widgets import CKEditorWidget
from codemirror2.widgets import AdminCodeMirrorEditor
from django.test import TestCase
from django.conf import settings
from django.core.cache import cache

from ..utils import get_domain, get_protocol, default_language, get_html_field_widget

__all__ = (
    'UtilsTestCase',
)

class UtilsTestCase(TestCase):
    def test_get_domain(self):
        site_name = get_domain()
        self.assertEqual(site_name, 'example.com')

    def test_get_protocol(self):
        self.assertEqual(get_protocol(), 'http')

        with self.settings(USE_HTTPS=True):
            self.assertEqual(get_protocol(), 'https')

    def test_default_language(self):
        # No parler settings
        with self.settings(PARLER_LANGUAGES={}):
            self.assertEqual(default_language(), settings.LANGUAGE_CODE)

        # With parler settings
        with self.settings(PARLER_LANGUAGES={'default': {'fallback': 'ru',}}):
            self.assertEqual(default_language(), 'ru')

    def test_get_html_field_widget(self):
        # No custom widget settings
        with self.settings(POSTIE_HTML_ADMIN_WIDGET={}):
            self.assertTrue(isinstance(get_html_field_widget(), AdminCodeMirrorEditor))

        # With custom widget settings
        with self.settings(POSTIE_HTML_ADMIN_WIDGET={
            'widget_module': 'ckeditor.widgets', 'widget': 'CKEditorWidget'
        }):
            self.assertTrue(isinstance(get_html_field_widget(), CKEditorWidget))



from typing import Dict

from django.core.management.base import BaseCommand
from django.conf import settings
from django.utils.translation import activate
from django.db import transaction

from postie.models import Template
from postie.const import POSTIE_TEMPLATE_CONTEXTS


class Command(BaseCommand):
    help = 'Creates templates that are not created'

    @transaction.atomic
    def handle(self, *args, **options):
        templates = Template.objects.all().values_list('event', flat=True)
        created = 0

        for event, context in POSTIE_TEMPLATE_CONTEXTS.items():
            if event in templates:
                continue

            self.create_blank_template(event, context)

            created += 1

        self.stdout.write(self.style.SUCCESS(f'Created {created} templates'))

    def create_blank_template(self, event: str, context: Dict):
        activate(settings.LANGUAGE_CODE)
        template_context = self.generate_template(event, context)

        template = Template.objects.create(**template_context)

        for code, verbose in settings.LANGUAGES:
            template.set_current_language(code)

            for field, value in template_context.items():
                setattr(template, field, value)

            template.save()

    def generate_template(self, event: str, context: Dict) -> Dict:
        vars = '<br>'.join(
            f'{{{{ {key} }}}}: {value}' for key, value in context.items()
        )

        return {
            'name': event.title().replace('_', ' '),
            'subject': event.title().replace('_', ' '),
            'html': f'<html><body>{vars}</body></html>',
            'plain': vars,
            'event': event,
        }



from django import test
from django.core.management import call_command

from django_dynamic_fixture import G

from ..models import Template
from postie import const


class TestCreateTemplatesCommandTestCase(test.TestCase):
    def test_command(self):
        contexts = {
            '1': {
                'var1': 'desc',
                'var2': 'desc 1',
                'var3': 'desc 2'
            },
            '2': {
                'var1': 'desc',
                'var2': 'desc 1',
                'var3': 'desc 2'
            },
        }
        Template.objects.all().delete()

        const.POSTIE_TEMPLATE_CONTEXTS = contexts

        self.assertEqual(Template.objects.count(), 0)
        call_command('create_templates')
        self.assertEqual(Template.objects.count(), 2)

    def test_command_already_created_templates(self):
        contexts = {
            '1': {
                'var1': 'desc',
                'var2': 'desc 1',
                'var3': 'desc 2'
            },
            '2': {
                'var1': 'desc',
                'var2': 'desc 1',
                'var3': 'desc 2'
            },
        }
        Template.objects.all().delete()
        G(Template, event='1')

        const.POSTIE_TEMPLATE_CONTEXTS = contexts

        self.assertEqual(Template.objects.count(), 1)
        call_command('create_templates')
        self.assertEqual(Template.objects.count(), 2)



from setuptools import find_packages, setup

pkj_name = 'postie'


with open('README.rst') as f:
    readme = f.read()

setup(
    name='django-postie',
    version='0.8.0',
    install_requires=[
        'django>=2',
        'django-ckeditor',
        'django-codemirror2',
        'django-model-utils',
        'django-parler',
        'django-solo',
        'tilda-wrapper-api'
    ],
    packages=find_packages(exclude=['tests']),
    url='https://gitlab.com/cyberbudy/django-postie',
    license='MIT',
    author='cyberbudy',
    author_email='cyberbudy@gmail.com',
    description='Django mailing through admin',
    long_description=readme,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3'
    ],
    include_package_data=True,
    python_requires='>3.7.0'
)



from pathlib import Path

from django.utils.translation import ugettext_lazy as _

from model_utils import Choices
import environ
env = environ.Env()

BASE_DIR = Path('..')
BASE_ROOT = BASE_DIR / '..'

SECRET_KEY = '!SECRET_KEY!'

DEBUG = True
TEMPLATE_DEBUG = DEBUG
DEBUG_PROPAGATE_EXCEPTIONS = True

ROOT_URLCONF = 'postie.tests.urls'

INSTALLED_APPS = (
    'postie',
    'postie.integrations.tilda',
    'solo',
    'codemirror2',
    'ckeditor',
    'parler',

    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.admin',
    'django.contrib.staticfiles',
)

MIDDLEWARE = MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR / 'templates'],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': (
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            )
        },
    },
]

DATABASES = {
    'default': env.db('DATABASE_URL', default='postgres://postgres:@localhost:5432/postie')

}

TIME_ZONE = 'UTC'
LANGUAGE_CODE = 'en'
USE_I18N = True
USE_L10N = True

SITE_ID = 1

STATIC_URL = '/static/'
STATIC_ROOT = BASE_ROOT / 'static'
MEDIA_URL = '/uploads/'
MEDIA_ROOT = BASE_ROOT / 'uploads'


POSTIE_TEMPLATE_CHOICES = Choices(
    ('1', '1'),
    ('2', '2'),
    ('3', '3'),
)

POSTIE_TEMPLATE_CONTEXTS = {
    '1': {
        'var1': 'desc',
        'var2': 'desc1',
        'var3': 'desc2'
    }
}

POSTIE_INSTANT_SEND = True
POSTIE_TASK_COUNTDOWN = 5

PARLER_LANGUAGES = {
    1: (
        {'code': 'en', },
        {'code': 'en-us', },
        {'code': 'it', },
        {'code': 'nl', },
    ),
    'default': {
        'fallback': 'en',             # defaults to PARLER_DEFAULT_LANGUAGE_CODE
        # the default; let .active_translations() return fallbacks too.
        'hide_untranslated': False,
    }
}

LANGUAGES = (
    ('en', _("English")),
    ('en-us', _("US English")),
    ('it', _('Italian')),
    ('nl', _('Dutch')),
    ('fr', _('French')),
    ('es', _('Spanish')),
)

LOCALE_PATH = [
    Path(__file__).parent.parent / 'locale'
]



from dataclasses import dataclass
from .models import Preferences


@dataclass(frozen=True)
class Credentials:
    public_key: str
    private_key: str
    project_id: int



def get_tilda_credentials() -> Credentials:
    preferences = Preferences.get_solo()

    return Credentials(
        public_key=preferences.public_key,
        private_key=preferences.private_key,
        project_id=preferences.project_id
    )



# Generated by Django 2.1 on 2020-07-08 18:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tilda', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='preferences',
            name='project_id',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Private key'),
        ),
    ]



from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from model_utils import Choices


__all__ = (
    'LETTER_STATUSES',
    'EVENTS',
    'POSTIE_TEMPLATE_CONTEXTS'
)


LETTER_STATUSES = Choices(
    ('draft', _('draft')),
    ('sent', _('Sent')),
    ('failed', _('Failed'))
)

EVENTS = getattr(settings, 'POSTIE_TEMPLATE_CHOICES', [])
POSTIE_TEMPLATE_CONTEXTS = getattr(settings, 'POSTIE_TEMPLATE_CONTEXTS', {})



from typing import Dict, List, Optional

from django.conf import settings

from .shortcuts import send_mail

__all__ = (
    'MailSendFormMixin',
)


class MailSendFormMixin:
    """
    Form mixin that sends email after form saving

    Attributes:
        email_template_event (str): Email template to use in sending.
            One of the keys of the POSTIE_TEMPLATE_CONTEXTS variable
    """

    email_template_event = None  # type: Optional[str]

    def get_email_context(self) -> Dict[str, str]:
        """
        Returns email context to use in rendering.

        Returns:
            dict: email context.
        """

        return {
            key: str(value) for key, value in self.cleaned_data.items()
        }

    def get_email_template_event(self) -> str:
        """
        Returns email template to render.

        Returns:
            str: email template name
        """

        return self.email_template_event

    def get_email_recipients(self) -> List[str]:
        """
        Return recipients to send email

        Returns:
            list: emails to send letter
        """

        raise NotImplementedError()

    def get_email_sender(self) -> str:
        """
        Returns sender email address

        Returns:
            str: sender email address
        """

        return settings.DEFAULT_FROM_EMAIL

    def send_email(self):
        """
        Sends an email.
        """

        send_mail(
            self.get_email_template_event(),
            self.get_email_recipients(),
            self.get_email_context(),
            self.get_email_sender()
        )

    def save(self, *args, **kwargs):
        """
        Overrides default save to send email after saving.
        """

        response = None

        if hasattr(super(), 'save'):
            response = super().save(*args, **kwargs)

        self.send_email()

        return response
